"use strict";

function logger(type, text, verbose) {
  if (!verbose || (verbose && verboseLogs === true)) {
    console.log(text)
    let message = [scannerID, "consoleMessage", type, text];
    process.send(message);
  }
}


async function exitProcess() {
  await new Promise((resolve) => {
    setTimeout(function () {
      resolve();
    }, 1000);
  });

  process.exit();
}

function finishScan() {

  let showButtons = false

  //Only restore scan buttons if not a folder watch scan (array)
  if (arrayOrPathSwitch === 1) {
    showButtons = true
  }

  try {
    let message = [scannerID, "finishScan", DB_id, showButtons];
    process.send(message);
  } catch (err) { }
}

process.on("message", function (message) {

  if (message[0] === "exitApproved") {
    exitProcess()
  }

});

process.on("uncaughtException", function (err) {
  console.error(err.stack);
  logger("fatal", "fatal:" + err, false);

    finishScan();
});

let t = {
  timers: {},
  start: (name) => {
    t.timers[name] = new Date()
  },
  get: (name) => {
    try {
      let timeElap = (((new Date()).getTime() - t.timers[name].getTime()) / 1000) + 's';
      return timeElap
    } catch (err) {
      return 'No timer'
    }
  }
}

t.start('fileScanner')

let scanArgs = JSON.parse(process.argv[2])
let {
  scannerID,
  DB_id,
  arrayOrPath,
  arrayOrPathSwitch,
  allowedContainers,
  mode,
  filePropertiesToAdd,
  homePath,
  scanTypes,
  foldersToIgnore,
  resBoundaries,
  verboseLogs
} = scanArgs;


process.env.scannerID = scannerID
logger("info", "Online", false);


const path = require("path");
const fs = require("fs");

let rootModules
if (fs.existsSync(path.join(process.cwd(), "/npm"))) {
  rootModules = path.join(process.cwd(), "/npm/node_modules/");
} else {
  rootModules = "";
}

const importFresh = require(rootModules + "import-fresh");
const scanFile = importFresh("./scanFile.js");
const isDocker = require(rootModules + "is-docker");

let filesInDB;
let foundCounter = 0;
let filesToScan = [];

try {

  if (mode === 0) {

    if (isDocker()) {
      logger("warn", "Filescanner in Docker", false);

      filesInDB = fs.readFileSync("/temp/" + scannerID + ".txt", "utf8");
      fs.unlinkSync("/temp/" + scannerID + ".txt");
    } else {
      filesInDB = fs.readFileSync(
        homePath + "/Tdarr/Data/" + scannerID + ".txt",
        "utf8"
      );
      fs.unlinkSync(homePath + "/Tdarr/Data/" + scannerID + ".txt");
    }

    filesInDB = filesInDB.split("\r\n");
  } else if (mode === 3) {
    if (isDocker()) {
      arrayOrPath = fs.readFileSync("/temp/" + scannerID + ".txt", "utf8");
      fs.unlinkSync("/temp/" + scannerID + ".txt");
    } else {
      arrayOrPath = fs.readFileSync(
        homePath + "/Tdarr/Data/" + scannerID + ".txt",
        "utf8"
      );
      fs.unlinkSync(homePath + "/Tdarr/Data/" + scannerID + ".txt");
    }

    arrayOrPath = arrayOrPath.split("\r\n");
    filesInDB = [];
  } else {
    filesInDB = [];
  }

} catch (err) {
  logger("error", "error:" + err, false);
  filesInDB = [];
}

if (arrayOrPathSwitch === 0) {

  for (let i = 0; i < arrayOrPath.length; i++) {
    if (
      checkContainer(arrayOrPath[i]) !== true ||
      isInIgnoredFolder(arrayOrPath[i]) === true
    ) {
      logger("warn", `File ${arrayOrPath[i]} does not meet container or ignored folder requirements.`, true);
      arrayOrPath.splice(i, 1);
      i--;
    }
  }

  //logger("info", `Launching FFprobe on these files: ${arrayOrPath}`);
  arrayOrPath = arrayOrPath.filter((row) => !row.includes("TdarrNew"));
  ffprobeLaunch(arrayOrPath);
}

if (arrayOrPathSwitch === 1) {
  //scanFiles'(DB_id,arrayOrPath,arrayOrPathSwitch) {
  // scanFilePath

  let validFolder = true
  try {
    validFolder = fs.lstatSync(arrayOrPath).isDirectory()
  } catch (err) {
    validFolder = false
  }

  if(validFolder === true){
    traverseDir(arrayOrPath);
  }

  function traverseDir(inputPathStem) {
    try {
      fs.readdirSync(inputPathStem).forEach((file) => {
        try {
          let fullPath = path.join(inputPathStem, file).replace(/\\/g, "/");
          if (fs.lstatSync(fullPath).isDirectory()) {
            try {
              traverseDir(fullPath);
            } catch (err) {
              console.error(err);
              logger("error", "error:" + err, false);
            }
          } else {
            fullPath = fullPath.replace(/\\/g, "/");
            logger("info", `Found: ${fullPath}`, true);

            foundCounter++;

            if (foundCounter % 10 === 0) {
              updateScanFound("Found:" + foundCounter + " " + getScannerMem())
            }

            //For Scan (Find new), check if file already in DB. Remove from array if so to increase performance.
            if (filesInDB.includes(fullPath)) {
              let idx = filesInDB.indexOf(fullPath);
              filesInDB.splice(idx, 1);
            } else {
              let checkCon = true;
              let ignored = false;

              try {
                checkCon = checkContainer(fullPath)
              } catch (err) {
                logger("error", "error:" + err, false);
              }
              try {
                ignored = isInIgnoredFolder(fullPath)
              } catch (err) {
                logger("error", "error:" + err, false);
              }

              if (
                checkCon === true &&
                ignored === false
              ) {
                filesToScan.push(fullPath);
              } else {
              }
            }
          } //end current file, go to next
        } catch (err) {
          console.error(err);
          logger("error", "error:" + err, false);
        }
      });
    } catch (err) {
      console.error(err);
      logger("error", "error:" + err, false);
    }
  }

  logger("info", "Sending files for extraction", false);
  foundCounter = 0;
  filesToScan = filesToScan.filter((row) => !row.includes("TdarrNew"));
  ffprobeLaunch(filesToScan);
}

function isInIgnoredFolder(filePath) {
  for (let i = 0; i < foldersToIgnore.length; i++) {
    if (
      foldersToIgnore[i].length >= 1 &&
      filePath.includes(foldersToIgnore[i])
    ) {
      return true;
    }
  }
  return false;
}

function checkContainer(newFile) {
  try {
    let fileType = path.extname(newFile).split(".").join("");

    for (let j = 0; j < allowedContainers.length; j++) {
      if (fileType.toLowerCase() === allowedContainers[j].toLowerCase()) {
        return true;
      }
    }
  } catch (err) {
    console.error(err.stack);
    logger("error", "error:" + err, false);
  }
  return false;
}


function ffprobeLaunch(filesToScan) {
  //logger("info", `ffprobeLaunch received these files:${filesToScan} `);

  async function processArray() {
    for (const item of filesToScan) {
      try {
        await runExtractions(item);
      } catch (err) {
        console.error(err.stack);
        logger("error", "error:" + err, false);
      }
    }

    logger("info", "Finished in " + t.get('fileScanner'), false);

    
    finishScan();
  }

  processArray();

  async function runExtractions(filepath) {
    if (fs.existsSync(filepath)) {
      try {
        let fileInfo = await scanFile(filepath, scanTypes)
        if (fileInfo.ffProbeData.result !== "error") {
          extractData(
            filepath,
            fileInfo
          );
        } else {
          extractDataError(filepath, fileInfo.ffProbeData.data);
        }
      } catch (err) {
        console.error(err.stack);
      }
    }
  }


  function extractDataError(filepath, err) {
    let thisFileObject = {};
    let container = path.extname(filepath).split(".").join("");
    thisFileObject.container = container.toLowerCase();
    thisFileObject.ffProbeRead = "error";

    let singleFileSize;
    try {
      singleFileSize = fs.statSync(filepath);
      singleFileSize = singleFileSize.size;
      let fileSizeInMbytes = singleFileSize / (1024 * 1024);
      thisFileObject.file_size = fileSizeInMbytes;
    } catch (err) { }

    thisFileObject.cliLog =
      "FFprobe was unable to extract data from this file. It is likely that the file is corrupt else FFprobe can't handle this file.";
    logger(
      "error",
      `FFprobe was unable to extract data from this file:${filepath}`, false);

    let obj = {
      HealthCheck: "Error",
      TranscodeDecisionMaker: "Queued",
      lastHealthCheckDate: new Date(),
      lastTranscodeDate: new Date(),
    };

    addFileToDB(filepath, thisFileObject, obj);
  }

  function extractData(filepath, fileInfo) {
    let thisFileObject = {};

    thisFileObject.meta = fileInfo.exifToolData.data;
    thisFileObject.mediaInfo = fileInfo.mediaInfo.data;
    thisFileObject.hasClosedCaptions = fileInfo.ccextractorData.data;

    logger("info", `Beginning extractData on:${filepath}`, true);
    let container = path.extname(filepath).split(".").join("");
    thisFileObject.container = container.toLowerCase();
    thisFileObject.ffProbeRead = "success";
    thisFileObject.ffProbeData = fileInfo.ffProbeData.data;
    logger("info", `Tagging ffprobe data:${filepath}`, true);
    let jsonString = JSON.stringify(fileInfo.ffProbeData.data);
    logger("info", `Tagging size data:${filepath}`, true);

    let singleFileSize;
    try {
      singleFileSize = fs.statSync(filepath);
      singleFileSize = singleFileSize.size;
      let fileSizeInMbytes = singleFileSize / (1024 * 1024);
      thisFileObject.file_size = fileSizeInMbytes;
    } catch (err) { }

    let bit_rate = undefined

    try {
      if (thisFileObject.meta && thisFileObject.meta.Duration) {
        bit_rate = (8 * singleFileSize) / parseFloat(thisFileObject.meta.Duration);
      } else if (thisFileObject.ffProbeData && thisFileObject.ffProbeData.streams[0] && thisFileObject.ffProbeData.streams[0]["duration"]) {
        bit_rate = (8 * singleFileSize) / parseFloat(thisFileObject.ffProbeData.streams[0]["duration"]);
      }
      thisFileObject.bit_rate = bit_rate;
    } catch (err) {
      logger("error", `Tagging bitrate data failed:${filepath}`, false);
    }

    //if (!!("video_codec_name" in thisFileObject)) {
    let exifToolVerdict
    try {
      exifToolVerdict = thisFileObject.meta.MIMEType.includes("video");
    } catch (err) {
      exifToolVerdict = true;
    }

    if (
      jsonString.includes('"codec_type":"video"') &&
      exifToolVerdict === true
    ) {
      logger("info", `Tagging video res:${filepath}`, true);
      try {
        let vidWidth = thisFileObject.ffProbeData.streams[0]["width"];
        let vidHeight = thisFileObject.ffProbeData.streams[0]["height"];
        let videoResolution = "";

        //    console.log(vidWidth, vidHeight)
        //Bounds +- 10%
        // 480p	     720	480		648	792		432	        528
        // 576p	    720	    576		648	792		518.4	    633.6
        // 720p	    1280	720		1152/864	1408		648	792
        // 1080p	1920	1080		1728	2112		972	1188
        // 4KUHD	3840	2160		3456	4224		1944	2376
        // DCI4K	4096	2160		3686.4	4505.6		1944	2376
        // 8KUHD	7680	4320		6912	8448		3888	4752

        if (
          vidWidth >= resBoundaries.res480p.widthMin &&
          vidWidth <= resBoundaries.res480p.widthMax &&
          vidHeight >= resBoundaries.res480p.heightMin &&
          vidHeight <= resBoundaries.res480p.heightMax
        ) {
          videoResolution = "480p";
        } else if (
          vidWidth >= resBoundaries.res576p.widthMin &&
          vidWidth <= resBoundaries.res576p.widthMax &&
          vidHeight >= resBoundaries.res576p.heightMin &&
          vidHeight <= resBoundaries.res576p.heightMax
        ) {
          videoResolution = "576p";
        } else if (
          vidWidth >= resBoundaries.res720p.widthMin &&
          vidWidth <= resBoundaries.res720p.widthMax &&
          vidHeight >= resBoundaries.res720p.heightMin &&
          vidHeight <= resBoundaries.res720p.heightMax
        ) {
          videoResolution = "720p";
        } else if (
          vidWidth >= resBoundaries.res1080p.widthMin &&
          vidWidth <= resBoundaries.res1080p.widthMax &&
          vidHeight >= resBoundaries.res1080p.heightMin &&
          vidHeight <= resBoundaries.res1080p.heightMax
        ) {
          videoResolution = "1080p";
        } else if (
          vidWidth >= resBoundaries.res4KUHD.widthMin &&
          vidWidth <= resBoundaries.res4KUHD.widthMax &&
          vidHeight >= resBoundaries.res4KUHD.heightMin &&
          vidHeight <= resBoundaries.res4KUHD.heightMax
        ) {
          videoResolution = "4KUHD";
        } else if (
          vidWidth >= resBoundaries.resDCI4K.widthMin &&
          vidWidth <= resBoundaries.resDCI4K.widthMax &&
          vidHeight >= resBoundaries.resDCI4K.heightMin &&
          vidHeight <= resBoundaries.resDCI4K.heightMax
        ) {
          videoResolution = "DCI4K";
        } else if (
          vidWidth >= resBoundaries.res8KUHD.widthMin &&
          vidWidth <= resBoundaries.res8KUHD.widthMax &&
          vidHeight >= resBoundaries.res8KUHD.heightMin &&
          vidHeight <= resBoundaries.res8KUHD.heightMax
        ) {
          videoResolution = "8KUHD";
        } else {
          videoResolution = "Other";
        }

        thisFileObject.video_resolution = videoResolution;
      } catch (err) {
        logger("error", `Tagging video res failed:${filepath}`, false);
      }

      thisFileObject.fileMedium = "video";
      thisFileObject.video_codec_name =
        thisFileObject.ffProbeData.streams[0]["codec_name"];
    } else if (jsonString.includes('"codec_type":"audio"')) {
      thisFileObject.fileMedium = "audio";
      thisFileObject.audio_codec_name =
        thisFileObject.ffProbeData.streams[0]["codec_name"];
    } else {
      thisFileObject.fileMedium = "other";
    }

    filePropertiesToAdd.cliLog += "\n";
    addFileToDB(filepath, thisFileObject, filePropertiesToAdd);
  }
}

function addFileToDB(filePath, FileObject, obj) {
  logger("info", `Sending add file to DB:${filePath}`, true);
  obj = JSON.parse(JSON.stringify(obj));
  FileObject._id = filePath;
  FileObject.file = filePath;
  FileObject.DB = DB_id;
  FileObject.lastPluginDetails = "none";
  FileObject.processingStatus = false;
  FileObject.createdAt = new Date();

  if (FileObject.file_size === undefined) {
    FileObject.file_size = 0;
  }

  if (FileObject.bit_rate === undefined) {
    FileObject.bit_rate = 0;
  }

  FileObject.statSync = fs.statSync(filePath);
  obj.history += addHistory(FileObject);

  function addHistory(row) {
    let histoyString = "-------------------------------------\n";
    histoyString += `Date:${row.createdAt.toISOString()}\n`;
    histoyString += `Size:${
      row.file_size !== undefined
        ? parseFloat((row.file_size / 1024).toPrecision(4))
        : 0
      }GB\n`;
    histoyString += "Streams:\n";

    if (row.ffProbeData && row.ffProbeData.streams) {
      let streams = row.ffProbeData.streams;
      streams = streams.map((row) => {
        let arr = [];
        return (
          `<tr><td><p>${row.codec_name}</p></td>` +
          `<td><p>${row.codec_type}</p></td>` +
          "" +
          `<td><p>${row.bit_rate !== undefined
            ? parseFloat((row.bit_rate / (1024 * 1024)).toPrecision(4)) + " Mbs"
            : "-"
          }</p></td>` +
          "" +
          `<td><p>${row.tags !== undefined && row.tags.language !== undefined
            ? row.tags.language
            : "-"
          }</p></td>` +
          "" +
          `<td><p>${row.tags !== undefined && row.tags.title !== undefined
            ? row.tags.title
            : "-"
          }</p></td></tr>`
        );
      });

      streams = streams.join("");
      streams = `<table className="streamsTable"><tbody>${streams}</tbody></table>`;
      histoyString += streams;
      return histoyString;
    } else {
      return "None";
    }
  }

  FileObject = { ...FileObject, ...obj };
  let message = [scannerID, "addFileToDB", JSON.stringify(FileObject)];
  process.send(message);
  foundCounter++;

  //usedJsHeapSize is the total amount of memory being used by JS objects including V8 internal objects, 
  //totalJsHeapSize is current size of the JS heap including free space not occupied by any JS objects.

  if (foundCounter % 10 === 0) {
    updateScanFound("Processing:" + foundCounter + "/" + filesToScan.length + " " + getScannerMem())
  }
}

function getScannerMem() {
  let mem = process.memoryUsage()
  return "Scanner mem: " + (mem.heapUsed / (1024 * 1024)).toFixed(1) + "MB/" + (mem.heapTotal / (1024 * 1024)).toFixed(1) + "MB"
}

process.on("message", (m) => {
  if (m[0] === 'exitThread') {
    logger("warn", `Cancel command received. Scanner exiting`, false);
    finishScan();
  }
})


function updateScanFound(update){
  let message = [
    scannerID,
    "updateScanFound",
    DB_id,
    update
  ];
  process.send(message);
}
