"use strict";


const path = require("path");
const fs = require("fs");

let rootModules
if (fs.existsSync(path.join(process.cwd(), "/npm"))) {
  rootModules = path.join(process.cwd(), "/npm/node_modules/");
} else {
  rootModules = "";
}

const importFresh = require(rootModules + "import-fresh");
const runFFprobe = importFresh("./runFFprobe.js");
const runExifTool = importFresh("./runExifTool.js");
const runCCExtractor = importFresh("./runCCExtractor.js");
const runMediaInfo = importFresh("./runMediaInfo.js");
const exiftool = require(rootModules + "exiftool-vendored").exiftool;

function logger(type, text) {
  if(process.env.scannerID){
    let message = [process.env.scannerID, "consoleMessage", type, text];
    process.send(message);
  }
}

process.on('exit', async function () {
  try {
    exiftool.end();
  } catch (err) { }

  await new Promise((resolve) => {
    setTimeout(() => {
      resolve();
    }, 2000);
  });
});


module.exports = async function scanFile(filepath, scanTypes) {
  
  let fileInfo = {
    ffProbeData: {
      data: null
    },
    mediaInfo: {
      data: null
    },
    exifToolData: {
      data: null
    },
    ccextractorData: {
      data: null
    }
  };

  try {
    //logger("info", `Running FFprobe on file ${filepath}`);
    await runFFprobe(filepath)
      .then((res) => {
        fileInfo.ffProbeData = res;
      })
      .catch((err) => {
        logger("warn", err);
        fileInfo.ffProbeData = {
          result: "error",
          data: "FFprobe encountered an error " + err,
        };
      });

      if (scanTypes.exifToolScan === true) {
        logger("info", "Running exiftool on file");
        await runExifTool(filepath, exiftool)
          .then((res) => {
            fileInfo.exifToolData = res;
          })
          .catch((err) => {
            logger("error", err)
            fileInfo.exifToolData = {
              result: "error",
              data: "Exiftool encountered an error " + err,
            };
          });
      }

      if (scanTypes.mediaInfoScan === true) {
        logger("info", "Running mediaInfo on file");
        await runMediaInfo(filepath)
          .then((res) => {
            fileInfo.mediaInfo = res;
          })
          .catch((err) => {
            logger("error", err);
            fileInfo.mediaInfo = {
              result: "error",
              data: "mediaInfo encountered an error " + err,
            };
          });
      }

      if (scanTypes.closedCaptionScan === true) {
        logger("info", "Running ccextractor on file");
        await runCCExtractor(filepath)
          .then((res) => {
            fileInfo.ccextractorData = res;
          })
          .catch((err) => {
            logger("error", err);
            fileInfo.ccextractorData = {
              result: "error",
              data: "CCextractor encountered an error " + err,
            };
          });
      }
  } catch (err) {
    console.log(err)
  }
  return fileInfo
}
