import { Maybe } from "./Maybe";
/**
 * Returns a "zone name" (used by `luxon`) that encodes the given offset.
 */
export declare function offsetMinutesToZoneName(offsetMinutes: Maybe<number>): Maybe<string>;
export declare const MaxTzOffsetHours = 14;
export declare function reasonableTzOffsetMinutes(tzOffsetMinutes: Maybe<number>): boolean;
export interface TzSrc {
    tz: string;
    src: string;
}
/**
 * Parse a timezone offset and return the offset minutes
 */
export declare function extractOffset(tz: Maybe<string>): Maybe<TzSrc>;
export declare function extractTzOffsetFromTags(t: {
    TimeZone?: string;
    OffsetTime?: string;
    /** time zone for DateTimeOriginal, "-08:00" */
    OffsetTimeOriginal?: string;
    /** time zone for CreateDate, "-08:00" */
    OffsetTimeDigitized?: string;
    /**
     * 1 or 2 values: 1. The time zone offset of DateTimeOriginal from GMT in
     * hours, 2. If present, the time zone offset of ModifyDate (which we ignore)
     * @see https://www.sno.phy.queensu.ca/~phil/exiftool/TagNames/EXIF.html
     */
    TimeZoneOffset?: number | number[] | string;
}): Maybe<TzSrc>;
export declare function inferLikelyOffsetMinutes(deltaMs: number): number;
export declare function extractTzOffsetFromUTCOffset(t: {
    DateTimeUTC?: string;
    GPSDateTime?: string;
    GPSDateStamp?: string;
    GPSTimeStamp?: string;
    SubSecDateTimeOriginal?: string;
    DateTimeOriginal?: string;
    SubSecCreateDate?: string;
    CreateDate?: string;
    SubSecMediaCreateDate?: string;
    MediaCreateDate?: string;
    DateTimeCreated?: string;
}): Maybe<TzSrc>;
//# sourceMappingURL=Timezones.d.ts.map